#!/bin/sh

if [ -r /proc/navico_platform/family_id ]; then
  FAMILYID="$(cut -f 2 /proc/navico_platform/family_id)"
fi

case $FAMILYID in
  Kiwi)
    TARGETUSERNAME=nos
    APPNAME=KiwiApp
    APPARGS="-qws -nomouse"
    ;;
  Sylvester)
    TARGETUSERNAME=nos
    APPNAME=IndigoApp
    APPARGS=""
    ;;
  *)
    TARGETUSERNAME=nos
    APPNAME=NOSApp
    APPARGS="-qws -nomouse"
    ;;
esac

HOMEDIR=/home/${TARGETUSERNAME}
BOOTMODE_FILE=${HOMEDIR}/boot_mode
LAUNCHER=/usr/bin/navico-applauncher
APP=$(readlink -f /usr/bin/$APPNAME)
APPLOGFILE=${HOMEDIR}/NOS/.log/$APPNAME
TESTAPP=/usr/sbin/TestApp
CLIENTCHECKER=/usr/sbin/ClientChecker
TIMEOUT=40
CONFIG_PARTITION=/etc/NOS
#MAP_PARTITION=/media/mmcblk0p2
TMP_DIR=/tmp/NOS
RUN_DIR=/var/run
BOOTKEYS=/dev/shm/bootkeys
CAPS="cap_net_admin,cap_sys_nice,cap_sys_time,cap_sys_tty_config+ep"

## The purpose of this section of initialization code is find the
## paths.ini file and read platform values for various settings, using
## reasonable defaults if it fails for any reason.
# TODO: turn this into a couple of calls to a generic function, parsepathsini()
SETTINGSINIPATH="${HOMEDIR}"
RESOURCESPATH=/usr/share/NOS/resources
#CHARTDATAPATH=/usr/share/NOS/charts

if [ -r "${HOMEDIR}/paths.ini" ]; then
  # SettingsIni=./
  if grep -qi ^SettingsIni= "${HOMEDIR}/paths.ini"; then
    SETTINGSINIPATH=$(grep -i ^SettingsIni= "${HOMEDIR}/paths.ini"|cut -d'=' -sf2)
    if [ -n "${SETTINGSINIPATH}" ]; then
      SETTINGSINIPATH="$(cd "${HOMEDIR}" && cd "${SETTINGSINIPATH}" && pwd)"
    fi
  fi
  # Resources=/usr/share/NOS/resources
  if grep -qi ^Resources= "${HOMEDIR}/paths.ini"; then
    RESOURCESPATH=$(grep -i ^Resources= "${HOMEDIR}/paths.ini"|cut -d'=' -sf2)
    if [ -n "${RESOURCESPATH}" ]; then
      RESOURCESPATH="$(cd "${HOMEDIR}" && cd "${RESOURCESPATH}" && pwd)"
    fi
  fi
  # ChartData=/usr/share/NOS/charts/
  #if grep -qi ^ChartData= "${HOMEDIR}/paths.ini"; then
  #  CHARTDATAPATH=$(grep -i ^ChartData= "${HOMEDIR}/paths.ini"|cut -d'=' -sf2)
  #  if [ -n "${CHARTDATAPATH}" ]; then
  #    CHARTDATAPATH="$(readlink ${CHARTDATAPATH%/})"
  #    MAP_PARTITION="${CHARTDATAPATH%/charts*}"
  #  fi
  #fi
fi

if [ -e /etc/navico-coredump.conf ]; then
    . /etc/navico-coredump.conf
fi

deleteflags() {
  rm $TMP_DIR/app-restart
  rm $TMP_DIR/poweroff
  rm $TMP_DIR/app-stop
}

movelogs() {
  if [ -e ${APPLOGFILE} ]; then
    [ -e ${APPLOGFILE}.2 ] && rm ${APPLOGFILE}.2
    [ -e ${APPLOGFILE}.1 ] && mv ${APPLOGFILE}.1 ${APPLOGFILE}.2
    mv ${APPLOGFILE} ${APPLOGFILE}.1
  fi
}

addlogentry() {
  echo $@
  if [ -e /dev/shm/enablecrashlog ]; then
    echo "rc.app: $@" >> ${APPLOGFILE}
    echo "$@" | syslogger -q -n rc.app
  fi
}

enabledebuglogin() {
  # Launch telnetd. It won't launch a second time, because it's bind to the port fails
  /usr/sbin/telnetd

  # Launch ssh
  [ -x /etc/rc.d/rc.sshd ] && /etc/rc.d/rc.sshd start
}

init_homedir() {
  mkdir -p $HOMEDIR.new
  cp -a /etc/skel/* $HOMEDIR.new/

  # By default, the root of new fs is owned by root. Change home dir to be owned by correct user
  chown -R ${TARGETUSERNAME}:users $HOMEDIR.new
  sync

  mv $HOMEDIR.new $HOMEDIR
  sync
}

restore_defaults() {
  # "Restore NOS defaults" - create a fresh home dir for $TARGETUSERNAME only
  # A full factory reset recreates the /home partition (refer rc.S)
  mv $HOMEDIR $HOMEDIR.old
  init_homedir
  rm -fr $HOMEDIR.old
  sync
}

read_boot_mode() {
  # Boot mode:
  #   1	TestApp
  #   2	NOSApp / normal boot (default)
  #   3	Client checker / burn-in
  BOOTMODE=$(grep -s '^[1-3]$' ${BOOTMODE_FILE} || echo 2)
}

# Create symlinks for compatibility with other products that have a dedicated
# partition for charts and splash screen images
detect_custom_splash() {
  if [ -d /etc/NOS/splash-images ]; then
    if [ ! -h /media/factorydata/splash-images ]; then
      rm -rf /media/factorydata/splash-images
      ln -s /etc/NOS/splash-images/ /media/factorydata/
    fi
  fi
}

launch_nosapp() {
  # Make sure the mcp251x module is loaded (used for third can controller).
  # NOTE: This may be un-loaded by the testapp so it can talk to the lcd spi
  case $FAMILYID in
    Kiwi) modprobe mcp251x ;;
  esac

  # Mediadeamon is stupid - its not really a deamon, but actually connects to NOSApp which is
  # acting as the server. So it needs to restart each time nosapp does
  if [ -x /etc/rc.d/rc.mediadaemon ]; then
    /etc/rc.d/rc.mediadaemon restart
  fi

  # Rotate logs on every start
  movelogs

  detect_custom_splash

  case $FAMILYID in
    Sylvester) 
        # Not using applauncher for Indigo.
        ${APP} ${APPARGS} &
        LAUNCHER_PID=$!
      ;;
    *)
      if [ -e /dev/shm/enablecrashlog ]; then
          su -c "${LAUNCHER} --timeout ${TIMEOUT} \
             --kill \
             --coredumpflag ${NAVICO_COREDUMP_FLAG} \
             --socket ${RUN_DIR}/app_monitor_daemon \
             -- ${APP} ${APPARGS} \
             2>&1 | syslogger -n NOSApp" \
             - ${TARGETUSERNAME} &
          LAUNCHER_PID=$!
      else
          su -c "${LAUNCHER} --timeout ${TIMEOUT} \
             --kill \
             --coredumpflag ${NAVICO_COREDUMP_FLAG} \
             --socket ${RUN_DIR}/app_monitor_daemon \
             -- ${APP} ${APPARGS}" \
             - ${TARGETUSERNAME} &
          LAUNCHER_PID=$!
      fi
      ;;
  esac

  sleep 1

  # Watch the nosapp log, and if it gets too big, rotate it
  ( while true; do
      sleep 60
      if [ -e ${APPLOGFILE} ] && [ "$(du ${APPLOGFILE} | awk '{ print $1 }')" -gt 256 ]; then
        movelogs

        # NOSApp responds to SIGUSR2 by closing and reopening its log file
        kill -USR2 $(pidof $APPNAME)
      fi
    done
  ) &
  LOG_WATCH_PID=$!

  if [ "$LAUNCHER_PID" ]; then
    wait $LAUNCHER_PID
  fi

  kill $LOG_WATCH_PID
  unset LOG_WATCH_PID

  read_boot_mode

  # check app stop flag
  if [ -f $TMP_DIR/app-stop ]; then
    addlogentry App Stop Requested
    deleteflags
    break

  # check app restart flag
  elif [ -f $TMP_DIR/app-restart ]; then
    addlogentry App Restart
    unset REBOOTING
    STOPCNT=0
    while pgrep $APPNAME; do
       sleep 1
       STOPCNT=$(($STOPCNT+1))
       if [ "$STOPCNT" -gt 15 ]; then
           if [ ! -e /dev/shm/debug ]; then
               /sbin/reboot
           fi
           REBOOTING=true
       fi
    done
    unset STOPCNT
    deleteflags

    if [ ! -z "$REBOOTING" ]; then
        break
    fi

  # check poweroff flag
  elif [ -f $TMP_DIR/poweroff ]; then
    addlogentry Power Off
    deleteflags
    /sbin/poweroff
    break

  elif [ "2" != "$BOOTMODE" ]; then
    echo Switching to BOOTMODE: $BOOTMODE

  else
    addlogentry App has died. Rebooting
    if [ ! -e /dev/shm/debug ]; then
      # Give log capture a chance to work
      if [ -e /dev/shm/enablecrashlog ]; then
        sleep 5
      fi
      /sbin/reboot
    fi
    break

  fi
}

start() {

  case "${FAMILYID}" in
    Kiwi)
      if [ ! -s $CONFIG_PARTITION/Config.ini ]; then
        mount -o remount,rw $CONFIG_PARTITION
        cat > $CONFIG_PARTITION/Config.ini <<EOF
[General]
INIVersion=2.1

[Product]
ProductId=Kiwi
ProductRegion=AMER

[ApplicationDefaults]
DefaultChartProvider=Navico
DefaultUnits=ImperialCoastal
DefaultVideoFormat=NTSC
DefaultTimeZone=-06:00
DefaultDateFormat=MM/dd/yyyy
DefaultTimeFormat=12hr
EOF
        sync
        mount -o remount,ro $CONFIG_PARTITION
      fi
      ;;
    Sylvester)
      # Sylvester just needs a file present.
      if [ ! -s $CONFIG_PARTITION/Config.dat ]; then
        mount -o remount,rw $CONFIG_PARTITION
        touch $CONFIG_PARTITION/Config.dat
        sync
        mount -o remount,ro $CONFIG_PARTITION
      fi
      ;;
  esac

  if [[ "$(getcap $APP | cut -d " " -f 3)" != "$CAPS" ]] || [[ "$(getcap $LAUNCHER | cut -d " " -f 3)" != "$CAPS" ]]; then
    echo "Setting application capabilities..."
    setcap $CAPS $APP
    setcap $CAPS $LAUNCHER
    sync
  fi

  # Make sure the temp directory actually exists
  if [ ! -d $TMP_DIR ]; then
    mkdir -p $TMP_DIR
  fi
  chown ${TARGETUSERNAME}:users ${TMP_DIR}

  # Make sure the home directory actually exists
  if [ ! -e $HOMEDIR ]; then
    init_homedir
  fi

  cd "$HOMEDIR"

  ( while true; do
    read_boot_mode

    if [ -r ${BOOTKEYS} ]; then
      case "${FAMILYID}" in
        Kiwi)
          if [ "$(cat ${BOOTKEYS} )" = "001" ]; then
            BOOTMODE=1
          fi
          ;;
        Sylvester)
          if [ "$(cat ${BOOTKEYS} )" = "108" ]; then
            BOOTMODE=1
          fi
          ;;
      esac

      mv ${BOOTKEYS} ${BOOTKEYS}.old 
    fi

    case ${BOOTMODE} in
      1)
        case "${FAMILYID}" in
          Sylvester)
            # The "TestApp" for Indigo is just a special mode of the application.
            mount -o remount,rw ${CONFIG_PARTITION}
            APPARGS=TEST
            launch_nosapp
            ;;
          *)
            # TestApp
            # Currently there are some requirements that must be met - refer http://nzwiki.marine.net.int/wiki/StingrayII/TestApp
            enabledebuglogin

            mount -o remount,rw ${CONFIG_PARTITION}

            # Stop mediadaemon, and ensure all the auto-mounted SD cards are un-mounted
            /etc/rc.d/rc.mediadaemon stop
            for MOUNTPOINT in /media/mmcblk*; do
                umount $MOUNTPOINT
            done
            unset MOUNTPOINT

            # Make sure the mcp251x module is loaded (used for third can controller).
            case $FAMILYID in
              Kiwi) modprobe mcp251x ;;
            esac


            # Still run as root, but wrap inside su to set environment correctly.
            # Pass the location of paths.ini to TestApp so it has a chance to be
            # smarter about where to find things
            if [ -r ${HOMEDIR}/paths.ini ]; then
              su -c "${TESTAPP} -qws -nomouse ${HOMEDIR}/paths.ini" -
            else
              su -c "${TESTAPP} -qws -nomouse" -
            fi
            EXITCODE=$?

            sync
            mount -o remount,ro ${CONFIG_PARTITION}

            # For TestApp exit codes, refer TestApp/Widgets/tSpecialModes.h
            case ${EXITCODE} in
              0|16)
                echo "Normal mode"
                restore_defaults # and implied: rm -f ${BOOTMODE_FILE}; sync
                ;;
              17)
                echo "Demo mode"
                # <TODO> ??!
                ;;
    #          18) - no 'burn-in' needed on kiwi
    #            echo "Burnin mode"
    #            echo 3 >${BOOTMODE_FILE}
    #            sync
    #            ;;
              18|19)
                echo "TestMode"
                echo 1 >${BOOTMODE_FILE}
                sync
                ;;
              20)
                echo "Reboot"
                /sbin/reboot
                exit
                ;;
               *)
                # Signals are 128+SIGNUM (eg 143 for SIGTERM)
                echo "$0: Unhandled exit code ${EXITCODE}"
                break
                ;;
            esac
          ;;
          esac
        ;;

      2)
        # NOSApp / normal boot
        launch_nosapp
        ;;

#      3)
#        # Client checker / burn-in
#        enabledebuglogin
#
#        mount -o remount,rw ${MAP_PARTITION}
#        mount -o bind ${MAP_PARTITION} /usr/share/NOS
#
#        # ClientChecker needs an IP address to function properly.  Spin.
#        echo "ClientChecker: Waiting for an IP address to appear on eth0."
#        unset IPADDR
#        while [ -z "${IPADDR}" ];
#        do
#          IPADDR=$(
#             ifconfig | \
#             grep -A1 '^eth0[^:]' | \
#             grep "inet addr:" | \
#             sed 's/.*inet addr://; s/ .*//'
#          )
#          sleep 1
#        done
#        unset IPADDR
#
#        # Still run as root, but wrap inside su to set environment correctly.
#        # Set QWS_MOUSE_PROTO to null, so that ClientChecker cannot be killed by pressing on the screen
#        su -c "cd ${HOMEDIR} && export QWS_MOUSE_PROTO="" && ${CLIENTCHECKER} -qws" -
#
#        sync
#        umount /usr/share/NOS
#        mount -o remount,ro ${MAP_PARTITION}
#
#        # Launch NOSApp so the factory can change BOOT_MODE via serial console.  Refer STP-161.
#        launch_nosapp
#        ;;
    esac
    done
  ) &
}

stop() {
  # TODO local kill_list="${APP} ${TESTAPP} ${CLIENTCHECKER}"

  if ! pgrep ${APP} > /dev/null 2>&1; then
    echo "App not running"
    exit 1
  else
    # Force script to exit when NOSApp killed
    # (signals are lost through applauncher)
    touch $TMP_DIR/app-stop

    pkill ${APP}
    while pgrep ${APP} > /dev/null 2>&1; do
      sleep 0.1
    done
  fi
}

case "$1" in
    start)
        start
        ;;
    stop)
        stop
        ;;
    restart)
        stop
        start
        ;;
    *)
        echo "Usage: $0 {start | stop | restart}"
        ;;
esac
